package bls

import (
	"github.com/herumi/bls-eth-go-binary/bls"
	"github.com/meshplus/bitxhub-kit/crypto"
	"github.com/meshplus/bitxhub-kit/types"
)

type Sign bls.Sign

var BLS_ETH crypto.KeyType = 9

func GenerateKeyPair(opt crypto.KeyType) (crypto.PrivateKey, error){
	var sec bls.SecretKey
	sec.SetByCSPRNG()
	return &PrivateKey{
		curve:     BLS_ETH,
		SecretKey: &sec,
	},nil
}

type PrivateKey struct {
	curve crypto.KeyType
	*bls.SecretKey
}

func (p *PrivateKey) Bytes() ([]byte, error) {
	return p.Serialize(), nil
}

func (p *PrivateKey) Type() crypto.KeyType {
	return p.curve
}

func (p *PrivateKey) Sign(digest []byte) ([]byte, error) {
	panic("implement me")
}

func (p *PrivateKey) PublicKey() crypto.PublicKey {
	return &PublicKey{
		curve:     p.curve,
		PublicKey: p.GetPublicKey(),
	}
}

type PublicKey struct {
	curve crypto.KeyType
	*bls.PublicKey
}

func (p *PublicKey) Bytes() ([]byte, error) {
	return p.Serialize(), nil
}

func (p *PublicKey) Type() crypto.KeyType {
	return p.curve
}

func (p *PublicKey) Address() (*types.Address, error) {
	panic("implement me")
}

func (p *PublicKey) Verify(digest []byte, sig []byte) (bool, error) {
	var sign bls.Sign
	err := sign.DeserializeHexStr(string(sig))
	if err != nil {
		return false, err
	}
	return sign.VerifyHash(p.PublicKey, digest), nil
}

func IsSign(aggSign *bls.Sign, aggDataSign *bls.Sign, selfKey *bls.PublicKey) bool {
	return bls.VerifyPairing(aggSign, aggDataSign, selfKey)
}
